/*
A simple utility for executing MDX queries on TM1 server
It reads connection properties from an XML file, tries to construct a view out of given MDX query and returns results as a csv files with (optional ) headers
Fill the xml  sample, then run
java tm1_mdx name_of_your_xml_file.xml

Author: Yuri Kudryavcev, mail@ykud.com
*/
import com.applix.tm1.*;
import java.io.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import java.util.ArrayList; 
import java.util.Random;
import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;


public class tm1_mdx
{
public static void main(String[] args) {
	try 
	{
		if (args.length == 0)
		{
			System.out.println ("Please supply a valid xml connection file.");
		}
		else
		{
			File file = new File(args[0]);
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			DocumentBuilder db = dbf.newDocumentBuilder();
			Document conf = db.parse(file);
			conf.getDocumentElement().normalize();
			//parsing configuration xml file
			boolean debug = Boolean.parseBoolean(conf.getElementsByTagName("debug").item(0).getChildNodes().item(0).getNodeValue());
			String AdminHost = "";
			if ( conf.getElementsByTagName("AdminHost").item(0).getChildNodes().getLength() != 0)
				{
					AdminHost = conf.getElementsByTagName("AdminHost").item(0).getChildNodes().item(0).getNodeValue();
				}
			else
				{
					System.out.println ("Please supply an Admin Host ip or dns name in xml file ");
					System.exit(-1);
				}
			String TM1ServerName = "";
			if ( conf.getElementsByTagName("tm1server").item(0).getChildNodes().getLength() != 0)
				{
					TM1ServerName = conf.getElementsByTagName("tm1server").item(0).getChildNodes().item(0).getNodeValue();
				}
			else
				{
					System.out.println ("Please supply a TM1 server name  in xml file ");
					System.exit(-1);
				}
			// user and password
			String user = "";
			if ( conf.getElementsByTagName("login").item(0).getChildNodes().getLength() != 0)
				{
					user = conf.getElementsByTagName("login").item(0).getChildNodes().item(0).getNodeValue();
				}
			else
				{
					System.out.println ("Please supply a user name for login in xml file ");
					System.exit(-1);
				}
			String password = "";	
			if ( conf.getElementsByTagName("password").item(0).getChildNodes().getLength()  != 0)
			{
				 password = conf.getElementsByTagName("password").item(0).getChildNodes().item(0).getNodeValue();
			}
			
			String namespace = "";
			if (conf.getElementsByTagName("namespace").item(0).getChildNodes().getLength() != 0)
				{
					namespace = conf.getElementsByTagName("namespace").item(0).getChildNodes().item(0).getNodeValue();
				}
			String mdxQuery = "";
			if ( conf.getElementsByTagName("MDX").item(0).getChildNodes().getLength() != 0)
				{
					mdxQuery = conf.getElementsByTagName("MDX").item(0).getChildNodes().item(0).getNodeValue();;
				}
			else
				{
					System.out.println ("Please supply an MDX query in xml file ");
					System.exit(-1);
				}
			String csvFilename = "test.csv";
			if (conf.getElementsByTagName("csv_file").item(0).getChildNodes().getLength() != 0)
				{
					csvFilename = conf.getElementsByTagName("csv_file").item(0).getChildNodes().item(0).getNodeValue();
				}
			Boolean needHeader = false;
			if (conf.getElementsByTagName("header").item(0).getChildNodes().getLength() != 0)
				{
					needHeader = Boolean.parseBoolean(conf.getElementsByTagName("header").item(0).getChildNodes().item(0).getNodeValue());
				}
			
			
			// Establishing connection with TM1 server
			TM1Bean fieldAdminServer;
			TM1Server fieldServer;

			fieldAdminServer = new TM1Bean();
			fieldAdminServer.setAdminHost(AdminHost);
			
			// trying to connect
			if (namespace == "")
			{ // TM1 security, straightforward
				fieldServer = fieldAdminServer.openConnection(TM1ServerName, user, password);
			}
			else 
			{	// CAM security, it's a bit more complicated 
				//https://www-304.ibm.com/support/docview.wss?uid=swg21418355 is a good sample of how to connect to Cognos-secured TM1
				TM1Val vArgs = TM1Val.makeArrayVal(3);
				TM1Val tm1ValNamespace = new TM1Val(namespace);
				TM1Val tm1ValUser = new TM1Val(user);
				TM1Val tm1Password =  new TM1Val(password);
				tm1Password.Encryption(true);
				vArgs.addToArray( 0, tm1ValNamespace);
				vArgs.addToArray(1, tm1ValUser);
				vArgs.addToArray(2,tm1Password);
				fieldServer = fieldAdminServer.openCAMConnectionWithNamespace(TM1ServerName, namespace, vArgs);
			}
			if (fieldServer.isError() )
			{
				System.out.println ("Couldn't connect to Admin Host: " + fieldServer.getErrorMessage());
			}
			else
			{
				// preparing the output file
				FileWriter fstream = new FileWriter( csvFilename);

				BufferedWriter csvFile = new BufferedWriter(fstream);

				// let's play with view's and MDX
				TM1MdxView mView = new TM1MdxView(fieldServer, mdxQuery);
				
				if (debug) {System.out.println ("Number of axis: " + mView.getAxisCount() );};
				// check the view structure
				if (debug) 
				{
					for (int j = 0; j < mView.getAxisCount(); j++)
						{
							System.out.println ("Number of dimensions on axis " + j + " is  " + mView.GetDimensionCount(j) );
							TM1MdxAxis ax = mView.GetAxis(j);
							for (int l = 0; l < ax.getDimensionCount() ; l++)
							{
								System.out.println ("Dimension " + l + " on axis " + j + " is  " + ax.GetDimensionName(l)  +" with " + ax.getTupleCount() + " tuples and has following properties");
								/*ArrayList  props = ax.GetPropertyNames(l); 
								for (int f = 0; f < props.size(); f++)
									{
										System.out.println (props.get(f)); // MEMBER_UNIQUE_NAME is a good property
									}
								*/	
								// let's scan elements
								
								for (int k = 0; k < ax.getTupleCount(); k ++)
									{
										System.out.println ( ax.GetAxisValue(k,l, "MEMBER_UNIQUE_NAME").getString());
									}
								
								
							}
						}
				}
				
				//outputing query results to csv file
				//axis 0 is columns, 1 is rows,  2  pages
				TM1MdxAxis pages = mView.GetAxis(2);
				TM1MdxAxis rows = mView.GetAxis(1);
				TM1MdxAxis columns = mView.GetAxis(0);
	
				if (needHeader)
				{
					
					// we start with printing pages info
					csvFile.write("Query dimension on pages:\n");
					for (int l = 0; l < pages.getDimensionCount() ; l++)
					{
						for (int k = 0; k < pages.getTupleCount(); k ++)
						{
							csvFile.write ( pages.GetDimensionName(l)  + ":," +  pages.GetAxisValue(k,l, "MEMBER_UNIQUE_NAME").getString() + '\n');
						}
									
									
					}
					//columns
					for (int l = 0; l < columns.getDimensionCount() ; l++)
					{
						//shifting it to incorporate all rows header
						csvFile.write (String.format(String.format("%%0%dd", rows.getDimensionCount()), 0).replace("0"," , ") + columns.GetDimensionName(l)  + ":,");
						for (int k = 0; k < columns.getTupleCount(); k ++)
						{
							csvFile.write ( columns.GetAxisValue(k,l, "MEMBER_UNIQUE_NAME").getString() + ',');
							
						}
						csvFile.write ('\n');		
									
					}
				}
				if (needHeader)
				{
					//row dimensions
					
					for (int l = 0; l < rows.getDimensionCount() ; l++)
					{
						csvFile.write (rows.GetDimensionName(l)  + ",");
					}
					csvFile.write ('\n');		
				}
				// outputing rows header and values 
				
				for (int k = 0; k < rows.getTupleCount(); k ++)
				{
					if (needHeader)
					{
						// row header			
						for (int l = 0; l < rows.getDimensionCount() ; l++)
							{
								csvFile.write ( rows.GetAxisValue(k,l, "MEMBER_UNIQUE_NAME").getString() + ',');	
							}
						//shifting 1 cell for aligning with columns
						csvFile.write(" ,");
					}
					//values
					for (int o = 0; o < columns.getTupleCount(); o++)
						{
							TM1Val cellValue = mView.GetCellValue(k,o);
							int type = cellValue.getType();
							switch (type)

							{

								case 0:

									if (debug)  {System.out.println ("Null Value");}

									csvFile.write(" ,");
									break;

								case 1:

									if (debug)  {System.out.println ("Real\\Float Value");}

									csvFile.write(Double.toString(cellValue.getReal()) + ',');
									break;

								case 2:

									if (debug)  {System.out.println ("String Value: " + cellValue.getString().toString());}

									csvFile.write(cellValue.getString().toString() + ',');
									break;

								case 3:

									if (debug)  {System.out.println ("int Value");}

									csvFile.write(cellValue.getInt() + ',');
									break;

								case 4:

									if (debug)  {System.out.println ("Boolean value");}

									break;

								case 5:

									if (debug)  {System.out.println ("Object value");}

									break;

								case 6:

									if (debug)  {System.out.println ("Error value");}

									break;

								case 7:

									if (debug)  {System.out.println ("Array value");}

									break;

								default:

									if (debug)  {System.out.println ("Default value");}

									break;

							/**/

						}
					}
					csvFile.write ('\n');	
				}
				
				csvFile.close();
			}
		}
	}
	catch (Exception e) 
	{
		e.printStackTrace();
	}
}

}
